//( firingneuron.hpp

#ifndef FIRINGNEURON_HPP
#define FIRINGNEURON_HPP


#include "timedependantobject.hpp"
#include "synapse.hpp"
#include "parameters.hpp"
#include <fstream>

//( FiringNeuron

/**
 * Neuron emitting spike
 *
 * @author Thomas
 **/
class FiringNeuron :
  public TimeDependantObject
{
public:
  /**
   * Default constructor
   *
   **/
  FiringNeuron();

  /**
   * Default destructor
   *
   **/
  virtual ~FiringNeuron();


public:
  /**
   *
   *
   * @return  true if the neuron emitted a spike
   *
   **/
  virtual bool isFiring() const = 0;

  /**
   *
   * Enumerate the different type of neurons
   *
   **/
  typedef enum
  {
    PyramidalCell,
    MyLifNeuron,
    Interneuron,
    NoiseNeuron,
    PoissonNeuron
  } NeuronType ;

  /**
   *
   *
   * @return  the type of this neuron
   *
   **/
  virtual NeuronType getNeuronType() const = 0;


public:
  /**
   *
   *
   * @return  a new neuron of the same nature of this one
   *
   **/
  virtual FiringNeuron * createNeuron(bool hebbLearning = false,
				      bool presynapticLearning = false,
				      bool postsynapticLearning = false) const = 0;


public:
  /**
   *
   * Set a stimulus adding to the noise
   *
   * @param   intensity  intensity of the stimulus
   *
   * @return  void
   *
   **/
  virtual void setStimulus(const double intensity = stimulusIntensity);

public:
  /**
   *
   * a vector recording the positions of the neurons firing at the present step in a layer to define
   * the first number represents the number of neurons firing, the following the index of these neurons
   *
   **/
  typedef vector<int> FiringPositions;

  /**
   *
   * set the index positions of the neurons firing in the layer to which this neuron is connected
   *
   * @param   efferentFiringPositions  vector of firing efferent neuron positions
   *
   * @return  void
   *
   **/
  void initPostSynapticFirings(FiringPositions & efferentFiringPositions);


public:
  /**
   *
   * Set a new index for this neuron
   *
   * @param   positionIndex  the new index
   *
   * @return  void
   *
   **/
  void setPositionIndex(double positionIndex);

  /**
   *
   *
   * @return  the current index of the neuron
   *
   **/
  double getPositionIndex() const;

  /**
   *
   * Set the noise for this neuron
   *
   * @param   noiseFlag  true if the neuron has to emitted some noise
   *
   * @return  void
   *
   **/
  void setNoise(bool noiseFlag);


public:
  /**
   *
   * Add a presynaptic connection
   *
   * @param   synapse  synapse where this neuron is the persynaptic neuron
   * @param   weight   weight of the connection
   *
   * @return  void
   *
   **/
  void addPresynapticConnection(Synapse * synapse, double weight);


public:
  /**
   *
   * print the weights to efferent neurons
   *
   * @param   weightsRaster  the stream to print data in
   *
   * @return  void
   *
   **/
  void printWeights(ofstream & weightsRaster);

public:
  /**
   *
   * print all presynaptic connection weights in a line
   *
   * @param   weightsStream  stream to write data in
   *
   * @return  void
   *
   **/
  void printAllWeights(ofstream & weightsStream);


public:
  /**
   *
   * return the weight of one of the connections
   *
   * @param   connectionIndex  index of the connectionwithin the presynapticConnections
   *
   * @return  the weight of the asked connection
   *
   **/
  double getWeight(int connectionIndex);


protected:
  /**
   *
   * Generate a spike for this neuron
   *
   *
   * @return  void
   *
   **/
  virtual void generateSpike();

  /**
   *
   * Send a spike to a synapse
   *
   * @param   synapse  the synapse where to send the spike
   * @param   weight   the weight of the connection
   *
   * @return  nothinh
   *
   **/
  virtual void sendSpike(Synapse * synapse, double weight);


protected:
  /**
   *
   * True if the neuron emits some noise
   *
   **/
  bool noiseFlag;


private:
  /**
   *
   * Position index of this neuron
   *
   **/
  double positionIndex;


public:
  /**
   *
   * Define the type of a presynaptic connection
   *
   **/
  class PresynapticConnection
  {
  public:
    /**
     *
     * Constructor
     *
     * @param   synapse  synapse making the connection
     * @param   weight   weight of this connection
     *
     *
     **/
    PresynapticConnection(Synapse * synapse, double weight)
    {
      this->synapse = synapse;
      this->weight = weight;
      numberOfConnection++;
    }

    /**
     *
     * function to modify the weight of the connection
     *
     * @param   weight   new connection weight
     *
     * @return void
     *
     **/
    void setConnectionWeight(double weight)
    {
      this->weight = weight;
    }

    /**
     *
     x     * Presynaptic neuron
     *
     **/
    Synapse * synapse;

    /**
     *
     * Weigth of the presynaptic neuron
     *
     **/
    float weight;

    /**
     *
     * Number of connection
     *
     **/
    static int numberOfConnection;

  };

protected:
  /**
   *
   * Define the type of list of firing neurons
   *
   **/
  typedef vector<PresynapticConnection *> PresynapticConnectionList; //list

  /**
   *
   * List of presynaptic connections for this synapse
   *
   **/
  PresynapticConnectionList presynapticConnections;


public:
  /**
   *
   * to know which of efferent neurons are firing (time saving variable)
   *
   **/
  FiringPositions * efferentFiringPositions;

}
;

//) FiringNeuron


#endif

//) firingneuron.hpp
