//( neuralnetwork.hpp
#ifndef NEURALNETWORK_HPP
#define NEURALNETWORK_HPP

#include <vector>
#include <fstream>
#include <string>
#include <limits>

using namespace std;

#include "lifneuron.hpp"
#include "synapse.hpp"

#include "synapseampa.hpp"
#include "synapsenmda.hpp"
#include "synapsegaba.hpp"

//#include "noiseneuron.hpp"
//#include "poissonneuron.hpp"
#include "pyramidalcell.hpp"
#include "mylifneuron.hpp"
//#include "interneuron.hpp"

#include <QtGui/QApplication>
//#include "mainwindow.h"
#include "globalVar.h"
#include "parameters.hpp"


/**
 * Neural Network
 *
 * @author Thomas
 **/
class NeuralNetwork :
  public TimeDependantObject
{
public:
  /**
   * Default constructor
   *
   **/
  NeuralNetwork();

  /**
   * Default destructor
   *
   **/
  virtual ~NeuralNetwork();

public:
  /**
   *
   * Define the type vector of pyramidal cell
   *
   **/
  typedef vector<FiringNeuron *> NeuronsVector;

  /**
   *
   *
   * @return  the vector of all the neurons in the network
   *
   **/
  const NeuronsVector * getNeuronsVector() const;

public:
  /**
   *
   * Define the type vector of pyramidal cell
   *
   **/
  typedef vector<Synapse *> SynapsesVector;

  /**
   *
   *
   * @return  the vector of all the neurons in the network
   *
   **/
  const SynapsesVector * getSynapsesVector() const;


  /**
   *
   * reset the firing positions vector
   *
   *
   * @return  void
   *
   **/
  void resetFiringPositions();

public:
//  MainWindow display_window;
//  QApplication app;

public:
  /**
   *
   * Print a raster in the output data file
   *
   * @param output  output used to write the data
   *
   * @return  void
   *
   **/
  void printRaster(ostream & output,int time);

  /**
   *
   * forward to afferent neurons the index of efferent neurons firing at the present time (in order to accelerate hebbian learning)
   *
   * @param   afferentVector  Vector of afferent neurons
   * @param   efferentNetwork  network of efferent neurons
   *
   * @return  void
   *
   **/
  void initPostFiringPositions(NeuronsVector & afferentVector, NeuralNetwork & efferentNetwork);


public:
  /**
   *
   * get the firing positions vector
   *
   *
   * @return  the corresponding vector
   *
   **/
  LifNeuron::FiringPositions & getFiringPositionsVector();

  /**
   *
   *
   * @return  the main vector of neurons
   *
   **/
  virtual NeuronsVector & getMainNeuronsVector() = 0;



protected:
  /**
   *
   * Compute one more step for this object
   *
   *
   * @return  void
   *
   **/
  void computeOneStep();


protected:
  /**
   *
   * Create a layer of neurons
   *
   * @param   numberOfNeurons   number of neurons to create
   * @param   model             model of neuron to create
   * @param   neuronsVector     layer of neuron to populate
   * @param   noiseFlag         true if the neurons has to create some noise
   * @param   hebbLearning      if we create a pyramidal cell or interneuron, will we perform learning on efferences ?
   *
   * @return  void
   *
   **/
  void createNeurons(const int numberOfNeurons,
		     const FiringNeuron & model,
		     NeuronsVector & neuronsVector,
		     bool noiseFlag = false,
		     bool hebbLearing = false,
		     bool presynapticLearning = false, bool postsynapticLearning = false);

  /**
   *
   * Create the postsynaptic connection for a neuron
   *
   * @param   neuronsLayer      layer of neurons using postsynaptic connections
   * @param   receptorType      type of receptor used in the synapse
   * @param   uniqueSynapse     true if the synapse is the same for every neurons in the layer
   * @param   externConnection  true if the connection are coming from the outside of the network
   *
   * @return  void
   *
   **/
  void createPostsynapticConnections(NeuronsVector & neuronsLayer,
				     Synapse::ReceptorType receptorType,
				     bool uniqueSynapse = false,
				     bool externConnection = false);

  /**
   *
   * Create connections between two set of neurons with only one
   * connection between neurons of each layer
   *
   * @param   neuronsVectorOrigin  the set of presynaptic neurons
   * @param   neuronsVectorTarget  the set of postsynaptic neurons
   * @param   receptorType         the type of receptor for the postsynaptic connections
   * @param   weight               weight of the connection
   *
   * @return  void
   *
   **/
  void createOneConnections(NeuronsVector & neuronsVectorOrigin,
			    NeuronsVector & neuronsVectorTarget,
			    const Synapse::ReceptorType receptorType,
			    double weight = 1);

  /**
   *
   * Create connections between two set of neurons
   *
   * @param   neuronsVectorOrigin  the set of presynaptic neurons
   * @param   neuronsVectorTarget  the set of postsynaptic neurons
   * @param   receptorType         the type of receptor for the postsynaptic connections
   * @param   weight               weight of the connection
   *
   * @return  void
   *
   **/
  void createConnections(NeuronsVector & neuronsVectorOrigin,
			 NeuronsVector & neuronsVectorTarget,
			 const Synapse::ReceptorType receptorType,
			 double weight = 1);





  /**
   *
   * Create connections between two set of neurons: with random weight from an uniform distribution
   * between weightMin and weightMax
   *
   * @param   neuronsVectorOrigin  the set of presynaptic neurons
   * @param   neuronsVectorTarget  the set of postsynaptic neurons
   * @param   receptorType         the type of receptor for the postsynaptic connections
   * @param   weightMin            minimum weight of the connections
   * @param   weightMax            maximum weight of the connections
   *
   * @return  void
   *
   **/
  void createRandomConnections(NeuronsVector & neuronsVectorOrigin,
			 NeuronsVector & neuronsVectorTarget,
			 const Synapse::ReceptorType receptorType,
			 double weightMin = numeric_limits<double>::min(), double weightMax = 1.0);


  /**
   *
   * create connections with weights pre-recorded in a file (during learning for example)
   *
   * @param   origin        vector of origin neurons
   * @param   target        vector of target neurons
   * @param   receptorType  type of synaptic receptor
   * @param   fileName      name of the file where data are recorded
   *
   * @return  void
   *
   **/
  void createRecordedConnections(NeuronsVector & origin,NeuronsVector & target,const Synapse::ReceptorType receptorType,const string & fileName, int originSize, int targetSize);


  /**
   *
   * Set an index to the neuron in the layer
   *
   * @param   neuronsLayer               the layer containing the neuron to index
   * @param   neuronsRangeDistribution   neuron range distribution
   * @return  void
   *
   **/
  void indexLayer(NeuronsVector & neuronsLayer, double neuronsRangeDistribution);

  /**
   *
   * Add a connection between two neurons using a synapse
   *
   * @param   presynapticNeuron  the neuron to add the connection for
   * @param   postsynapticNeuron the neuron to add the connection for
   * @param   synapse            synapse used for the connection
   * @param   weight             weight of the connection
   *
   * @return  void
   *
   **/
  void addConnection(FiringNeuron * presynapticNeuron,
		     LifNeuron * postsynapticNeuron,
		     Synapse * synapse, double weight);


protected:
  /**
   *
   * List of all the neurons in the network
   *   computeStimulus();
   **/
  NeuronsVector neuronsVector;


protected:

  /**
   *
   * Model of my custom LIF neuron (Xavier)
   *
   **/
  static MyLifNeuron myLifNeuronModel;

  /**
   *
   * Model of a pyramidal cell
   *
   **/
  static PyramidalCell pyramidalCellModel;

  /**
   *
   * Model of an interneuron
   *
   **/
 // static Interneuron interneuronModel;

  /**
   *
   * Model of neuron generating poisson noise
   *
   **/
 // static PoissonNeuron noiseNeuronModel;

  /**
   *
   * List of all the synapses in this network
   *
   **/
  SynapsesVector synapsesVector;

  /**
   *
   * the vector of firing neuron positions
   *
   **/
  FiringNeuron::FiringPositions firingPositions;

  /**
   *
   * Number of connections in the network
   *
   **/
  int numberOfConnections;

  /**
   *
   * Last average in this network computed
   *
   **/
  double lastAverage;

  /**
   *
   * Variable used to compute the current average
   *
   **/
  bool currentAverage;

}
;

//) NeuralNetwork


#endif

//) neuralnetwork.hpp
