//( synapse.hpp

#ifndef SYNAPSE_HPP
#define SYNAPSE_HPP


#include <list>
#include <vector>

using namespace std;

#include "timedependantobject.hpp"


class FiringNeuron;
class LifNeuron;

//( Synapse

/**
 * Synapse between two neurons
 *
 * @author Thomas
 **/
class Synapse  :
  public TimeDependantObject
{
public:
  /**
   *
   * Type of receptors for a synapse
   * 
   **/
  enum ReceptorType 
    {
      AMPAReceptor,
      NMDAReceptor,
      GABAReceptor
    };

  /**
   *
   * Constructor of the Synapse
   * 
   * @param   receptor            receptor used in this synapse
   *
   *
    **/
  Synapse(ReceptorType receptor);

  /**
   * Default destructor
   *
   **/
  ~Synapse();


public:
  /**
   *
   *
   * @return  the current intensity receives by the receptors of the synapse
   *
   **/
  double getGatingVariable() const;

  /**
   *
   *
   * @return  the type of receptor used for this synapse
   *
   **/
  ReceptorType getReceptorType() const;

  /**
   *
   *
   * @return  the reference potential of the receptors
   *
   **/
  virtual double getReceptorReferencePotential() const = 0;

  /**
   *
   *
   * @param   postsynapticNeuron  the postsynaptic neuron of this synapse
   *
   * @return  the leak conductance of the receptors
   *
   **/
  virtual double getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const = 0;


public:
  /**
   *
   * Add a presynaptic connection to this synapse
   * 
   * @param   presynapticNeuron   the presynaptic neuron
   * @param   postsynapticNeuron  the postsynaptic neuron
   * @param   weight              weight associated with the connection
   *
   * @return  void
   *
   **/
  void addConnection(FiringNeuron * presynapticNeuron, 
		     LifNeuron * postsynapticNeuron,
		     double weight = 1);

public:
  /**
   *
   *
   * @return  true if the synapse is recurrent in the network
   *
   **/
  bool isRecurrent() const;

  /**
   *PresynapticConnectionsList
   * Set if the synapse is recurrent in the network or not
   * 
   * @param   recurrent  the new value
   *
   * @return  void
   *
   **/
  void setRecurrent(bool recurrent);


public:
  /**
   *
   *
   * @return  the weight of this synapse
   *
   **/
  double getWeight() const;

  /**
   *
   * Set the weight of this synapse
   * 
   * @param   weight  the new weight of this synapse
   *
   * @return  void
   *
   **/
  void setWeight(double weight);


public:
  /**
   *
   * A presynaptic neuron calls this function to prevent this synapse
   * it has emitted a spike
   * 
   * @param   weight  weight of the connection with this neuron
   *
   * @return  void
   *
   **/
  void sendSpike(double weight);

  /**
   *
   *
   * @return  the sum of firings presynaptic neurons
   *
   **/
  double getSumFiringNeurons() const;


public:
  /**
   *
   * Get the intensity generated by this synapse
   * 
   * @param   postsynapticNeuron  the postsynaptic neuron of this synapse
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the intensity computed (ampere)
   *
   **/
  virtual double getGeneratedIntensity(const LifNeuron * postsynapticNeuron, 
				       RungeKutta rungeKutta = none) const;


public:
  /**
   *
   * Create a new synapse of the same model 
   * 
   *
   * @return  the newly created synapse
   *
   **/
  Synapse * createSynapse() const;


public:
  /**
   *
   * Provides thw efferent neuron 
   * 
   *
   * @return  the efferent neuron
   *
   **/
  LifNeuron * getEfferentNeuron(); 

protected:
  /**
   *
   * Create a new synapse of the same model 
   * 
   *
   * @return  the newly created synapse
   *
   **/
  virtual Synapse * createNewSynapse() const = 0;


protected:
  /**
   *
   * Compute one more step for this object
   * 
   *
   * @return  void
   *
   **/
  void computeOneStep();


protected:
  /**
   *
   * The weight of this synapse
   * 
   **/
  double weight;

  /**
   *
   * True if this synapse is recurrent in the current neural network
   * 
   **/
  bool recurrent;

  /**
   *
   * The sum of firing neurons
   * 
   **/
  double sumOfFiringNeurons;

protected:
  /**
   *
   * The neuron to which this synapse is connected
   * 
   **/
  LifNeuron * efferent;

private:
  /**
   *
   * Type of receptor used for this synapse
   * 
   **/
  const ReceptorType receptorType;

}
;

//) Synapse


#endif

//) synapse.hpp
