//( synapsenmda.hpp

#ifndef SYNAPSENMDA_HPP
#define SYNAPSENMDA_HPP


#include "synapse.hpp"


//( SynapseNMDA

/**
 * Synapse using NMDA receptors
 *
 * @author Thomas
 **/
class SynapseNMDA :
  public Synapse
{
public:
  /**
   *
   * Constructor of the Synapse
   * 
   *
   **/
  SynapseNMDA();

  /**
   * Default destructor
   *
   **/
  ~SynapseNMDA();

public:
  /**
   *
   *
   * @return  the reference potential of the receptors
   *
   **/
  double getReceptorReferencePotential() const;

  /**
   *
   *
   * @param   postsynapticNeuron  the postsynaptic neuron of this synapse
   *
   * @return  the leak conductance of the receptors
   *
   **/
  double getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const;


protected:
  /**
   *
   * Create a new synapse of the same model 
   * 
   *
   * @return  the newly created synapse
   *
   **/
  Synapse * createNewSynapse() const;


public:
  /**
   *
   * Get the intensity generated by a list of synapse for a neuron
   * using the same type of receptors
   *
   * @param   postsynapticNeuron  the postsynaptic neuron of this synapse 
   *
   * @return  the intensity computed (ampere)
   *
   **/
  double getGeneratedIntensity(const LifNeuron * postsynapticNeuron) const;


protected:
  /**
   *
   * Compute one more step for this object
   * 
   *
   * @return  void
   *
   **/
  void computeOneStep();

  /**
   *
   * Compute the derivate of the equation modeling the behaviour of
   * this neurons
   * 
   * @param   currentValue  the last value of the function
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the result of the computation
   *
   **/
  double computeDerivate(double currentValue, RungeKutta rungeKutta);


private:
  /**
   *
   * Class used to compute the x function of the NMDA receptor
   * 
   **/
  class RiseFunction :
    public TimeDependantObject
  {
  public:
    /**
     *
     * Constructor
     *
     * @param   parent   the synapse using an instance of this classe
     * 
     **/
    RiseFunction(Synapse * parent);


  protected:
    /**
     *
     * Compute one more step for this object
     * 
     *
     * @return  void
     *
     **/
    void computeOneStep();

    /**
     *
     * Compute the derivate of the equation modeling the behaviour of
     * this neurons
     * 
     * @param   currentValue  the last value of the function
     * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
     *
     * @return  the result of the computation
     *
     **/
    double computeDerivate(double currentValue, RungeKutta rungeKutta);


  private:
    /**
     *
     * Synapse using this instance
     * 
     **/
    Synapse * parent;
  };

//  public:
//    /**
//     *
//     * Class used to compute the x function of the NMDA receptors
//     * 
//     **/
//    RiseFunction riseFunction;
  
//  public: 
//    RiseFunction getRiseFunction() 
//    {
//      return riseFunction;
//    }
    
}
;

//) SynapseNMDA


#endif

//) synapsenmda.hpp
